@@ -13,7 +13,7 @@ from .fields import FieldEx
 
 class CrispyFormMixin:
     Meta: object
-    fields: list
+    fields: dict
     submit_class = 'btn-success modal-submit'
     cancel_class = 'btn-secondary modal-cancel'
     delete_class = 'btn-danger modal-delete'
@@ -27,11 +27,12 @@ class CrispyFormMixin:
 
     def __init__(self, *args, pk=None, no_buttons=None, modal_title=None, form_setup=None, slug=None,
                  request_user=None, form_id=None, process=None, layout_field_params=None, layout_field_classes=None,
-                 helper_class=HorizontalHelper, **kwargs):
+                 helper_class=HorizontalHelper, progress_bar=None, header_html=None, clean=None, **kwargs):
         self.supplied_kwargs = locals()
         self.no_buttons = self.get_defaults('no_buttons')
         self.modal_title = self.get_defaults('modal_title')
         self._form_id = self.get_defaults('form_id')
+        self.progress_bar = self.get_defaults('progress_bar')
 
         self.user = request_user
         self.slug = slug
@@ -47,6 +48,8 @@ class CrispyFormMixin:
         self.mode = []
         self.triggers = {}
         self.trigger_fields = {}
+        self.header_html = [HTML(header_html)] if header_html else []
+        self.clean_method = clean
         super().__init__(*args, **kwargs)
         self.setup_modal(*args, **kwargs)
 
@@ -61,18 +64,23 @@ class CrispyFormMixin:
         if self.form_setup:
             return self.form_setup(self, *args, **kwargs)
 
-    def submit_button(self, css_class=submit_class, button_text='Submit'):
-        return self.button(button_text, 'post_modal', css_class)
+    def submit_button(self, css_class=submit_class, button_text='Submit', **kwargs):
+        if self.progress_bar:
+            return self.button('submit', {'function': 'post_modal',
+                                          'options': {'progress': {'selector': f'#file_progress_bar'}}}, css_class,
+                               **kwargs)
+        else:
+            return self.button(button_text, 'post_modal', css_class, **kwargs)
 
-    def delete_button(self, css_class=delete_class):
+    def delete_button(self, css_class=delete_class, **kwargs):
         if self.instance.pk is not None:
-            return self.button('Delete', {'function': 'post_modal', 'button': 'delete'}, css_class)
+            return self.button('Delete', {'function': 'post_modal', 'button': 'delete'}, css_class, **kwargs)
 
-    def cancel_button(self, css_class=cancel_class):
-        return self.button('Cancel', 'close', css_class)
+    def cancel_button(self, css_class=cancel_class, **kwargs):
+        return self.button('Cancel', 'close', css_class, **kwargs)
 
-    def view_edit_button(self, css_class=edit_class):
-        return self.button('Edit', {'function': 'post_modal', 'button': 'make_edit'}, css_class)
+    def view_edit_button(self, css_class=edit_class, **kwargs):
+        return self.button('Edit', {'function': 'post_modal', 'button': 'make_edit'}, css_class, **kwargs)
 
     def button(self, title, commands, css_class, **kwargs):
         if self.no_buttons:
@@ -120,6 +128,22 @@ class CrispyFormMixin:
     def field_section(self, *args):
         return Div(*args, css_class=self.helper.field_class)
 
+    def format_layout_fields(self, *layout_fields):
+        if len(layout_fields) == 1 and isinstance(layout_fields[0], (list, tuple)):
+            layout_fields = layout_fields[0]
+        fields = []
+        for f in layout_fields:
+            if isinstance(f, str):
+                field_args = {}
+                if self.layout_field_params and f in self.layout_field_params:
+                    field_args.update(self.layout_field_params[f])
+                field_args.update(getattr(self.fields[f].widget, 'crispy_kwargs', {}))
+                # Class can be set in widget otherwise use FieldEx
+                fields.append(getattr(self.fields[f].widget, 'crispy_field_class', FieldEx)(f, **field_args))
+            else:
+                fields.append(f)
+        self.helper.layout = Layout(*self.header_html, *fields)
+
     def setup_modal(self, *args, **kwargs):
         self.helper = self.helper_class(self)
         self.helper.form_id = self.form_id
@@ -130,19 +154,11 @@ class CrispyFormMixin:
                     self.fields[a].required = self.layout_field_params[a].pop('required')
         if layout:
             if isinstance(layout, (tuple, list)):
-                self.helper.layout = Layout(*layout)
+                self.format_layout_fields(*layout)
             else:
                 self.helper.layout = Layout(layout)
         else:
-            fields = []
-            for f in self.fields:
-                field_args = {}
-                if self.layout_field_params and f in self.layout_field_params:
-                    field_args.update(self.layout_field_params[f])
-                field_args.update(getattr(self.fields[f].widget, 'crispy_kwargs', {}))
-                # Class can be set in widget otherwise use FieldEx
-                fields.append(getattr(self.fields[f].widget, 'crispy_field_class', FieldEx)(f, **field_args))
-            self.helper.layout = Layout(*fields)
+            self.format_layout_fields(*self.fields.keys())
         if getattr(self.helper, 'fields_wrap_class', None):
             # noinspection PyUnresolvedReferences
             self.helper[:].wrap_together(Div, css_class=self.helper.fields_wrap_class)
@@ -171,6 +187,17 @@ class CrispyFormMixin:
         self.triggers[field] = conditions
         self.trigger_fields.setdefault(field, []).append(trigger)
 
+    def reorder_fields(self, *key_order):
+        if len(key_order) == 1 and isinstance(key_order[0], (list, tuple)):
+            key_order = key_order[0]
+        self.fields = {k: self.fields[k] for k in key_order}
+
+    def clean(self):
+        cleaned_data = super().clean()
+        if self.clean_method:
+            self.clean_method(self, cleaned_data)
+        return cleaned_data
+
     def __str__(self):
         if self.triggers:
             for f, triggers in self.trigger_fields.items():
