@@ -1,4 +1,5 @@
 import json
+from base64 import urlsafe_b64encode
 from ajax_helpers.templatetags.ajax_helpers import button_javascript
 from django.urls import reverse, resolve, NoReverseMatch
 from django.template.loader import render_to_string
@@ -14,6 +15,28 @@ modal_buttons = {
     'delete': '<i class="fas fa-trash"></i>',
 }
 
+progress_bar_html = '''
+    <div class="progress" style="margin-top: 5px;">
+        <div id='file_progress_bar{}' class="progress-bar{}" role="progressbar" aria-valuenow="0" 
+            aria-valuemin="0" aria-valuemax="100" style="width: 0%">
+        </div>
+    </div>
+'''
+
+
+def base64_json(slug_dict):
+    return urlsafe_b64encode(json.dumps(slug_dict).encode("utf8")).decode("ascii")
+
+
+def progress_bar(progress_id=None, css=''):
+    if progress_id is not None:
+        progress_id = '_' + str(progress_id)
+    else:
+        progress_id = ''
+    if css:
+        css = ' ' + css
+    return progress_bar_html.format(progress_id, css)
+
 
 def make_slug(*args, make_pk=False):
     slug = ''.join([str(a) for a in args])
@@ -22,23 +45,31 @@ def make_slug(*args, make_pk=False):
     return slug
 
 
-def show_modal(modal_name, *args, datatable=False, href=False, button=None, button_classes='btn btn-primary mx-1',
-               row=False):
+def show_modal(modal_name, *args, base64=False, datatable=False, href=False, button=None,
+               button_classes='btn btn-primary mx-1', row=False, font_awesome=None):
     try:
         javascript = f"django_modal.show_modal('{reverse(modal_name, args=[DUMMY_SLUG])}')"
     except NoReverseMatch:
         javascript = f"django_modal.show_modal('{reverse(modal_name)}')"
-    slug = make_slug(*args)
+    if base64:
+        slug = base64_json(base64)
+    else:
+        slug = make_slug(*args)
     if datatable:
-        if slug:
-            slug += '-'
-        slug += 'pk-%ref%'
-        if row:
-            slug += '-row-%row%'
+        if base64:
+            slug = '%ref%'
+        else:
+            if slug:
+                slug += '-'
+            slug += 'pk-%ref%'
+            if row:
+                slug += '-row-%row%'
     if href:
         javascript = 'javascript:' + javascript
-    if button:
+    if button is not None:
         button_text = modal_buttons.get(button, button)
+        if font_awesome:
+            button_text = f'<i class="{font_awesome}"></i> {button_text}'
         javascript = f'<a {css_classes(button_classes)} href="javascript:{javascript}">{button_text}</a>'
     if not slug:
         slug = '-'
@@ -73,7 +104,7 @@ def modal_button(title, commands, css_class='btn-primary'):
     else:
         params = commands
     return mark_safe(f'''<button onclick='django_modal.process_commands_lock({json.dumps(params)})' 
-            class="btn {css_class}">{title}</button>''')
+            class="btn {css_class}" type="button">{title}</button>''')
 
 
 def modal_button_method(title, method_name, css_class='btn-primary', **kwargs):
@@ -95,9 +126,11 @@ def modal_delete_javascript(url_name, pk):
     return mark_safe(button_javascript('delete', url_name=url_name, url_args=[pk]).replace('"', "'"))
 
 
-def reverse_modal(modal_name, slug='-'):
+def reverse_modal(modal_name, slug='-', base64=None):
+    if base64:
+        slug = base64_json(base64)
     try:
-        return reverse(modal_name, kwargs={'slug': slug})
+        return reverse(modal_name, args=[slug])
     except NoReverseMatch:
         if slug == '-':
             return reverse(modal_name)
@@ -105,8 +138,9 @@ def reverse_modal(modal_name, slug='-'):
             raise NoReverseMatch
 
 
-def ajax_modal_redirect(modal_name, slug='-'):
-    return [{'function': 'close'}, {'function': 'show_modal', 'modal': reverse_modal(modal_name, slug)}]
+def ajax_modal_redirect(modal_name, slug='-', base64=None):
+    return [{'function': 'close'}, {'function': 'show_modal', 'modal': reverse_modal(modal_name, slug=slug,
+                                                                                     base64=base64)}]
 
 
 def ajax_modal_replace(request, modal_name=None, modal_class=None, slug='-', ajax_function='overwrite_modal', **kwargs):
