@@ -1,6 +1,7 @@
+import base64
+import binascii
 import json
 import inspect
-import collections
 from django.forms.fields import Field
 from django.forms.models import modelform_factory, fields_for_model
 from django.http import HttpResponse, JsonResponse
@@ -13,15 +14,12 @@ from django.urls import reverse
 from django.utils.safestring import mark_safe
 from django.utils.decorators import method_decorator
 
-from crispy_forms.utils import render_crispy_form
-
 from ajax_helpers.mixins import AjaxHelpers
 
+from . import processes
 from .forms import ModelCrispyForm
 from .helper import render_modal, modal_button, modal_button_group, ajax_modal_redirect, modal_button_method, \
     ajax_modal_replace
-from .processes import PROCESS_CREATE, PROCESS_VIEW, PROCESS_EDIT, PROCESS_DELETE, \
-    PROCESS_EDIT_DELETE, PERMISSION_OFF, PERMISSION_DISABLE, user_has_perm, get_process, modal_url_type
 
 
 class ModalException(Exception):
@@ -34,7 +32,7 @@ class BaseModalMixin(AjaxHelpers):
     button_group_class = None
     button_container_class = None
     menu_config = {'href_format': "javascript:django_modal.show_modal('{}')"}
-    ajax_commands = ['button', 'select2']
+    ajax_commands = ['button', 'select2', 'ajax']
     button_group_css = None
     size = 'lg'
     no_parent_template = 'django_modals/blank_page_form.html'
@@ -45,16 +43,9 @@ class BaseModalMixin(AjaxHelpers):
             self.modal_mode = True
         self.slug = {}
 
-    @staticmethod
-    def permission(process=None):
-        return True
-
     def get_context_data(self, **kwargs):
-        if hasattr(super(), 'get_context_data'):
-            # noinspection PyUnresolvedReferences
-            context = super().get_context_data(**kwargs)
-        else:
-            context = {}
+        # noinspection PyUnresolvedReferences
+        context = super().get_context_data(**kwargs) if hasattr(super(), 'get_context_data') else {}
         context.update({'request': self.request, 'slug': self.slug})
         context['modal_url'] = kwargs.get('modal_url', self.request.get_full_path())
         context['no_header_x'] = getattr(self, 'no_header_x', None)
@@ -64,7 +55,7 @@ class BaseModalMixin(AjaxHelpers):
         return context
 
     def split_slug(self, kwargs):
-        if 'slug' in kwargs:
+        if 'slug' in kwargs and kwargs['slug'] != '-':
             s = kwargs['slug'].split('-')
             if len(s) == 1:
                 self.slug['pk'] = s[0]
@@ -76,8 +67,19 @@ class BaseModalMixin(AjaxHelpers):
     def process_slug_kwargs(self):
         return True
 
+    def split_base64(self, kwargs):
+        if 'base64' in kwargs:
+            try:
+                base64_data = json.loads(base64.urlsafe_b64decode(self.kwargs['base64']))
+            except binascii.Error:
+                raise ModalException('Error in modal base64 decode - This modal requires base64 not a slug')
+            if not isinstance(base64_data, dict):
+                base64_data = {'base64': base64_data}
+            self.slug.update(base64_data)
+
     def dispatch(self, request, *args, **kwargs):
         self.split_slug(kwargs)
+        self.split_base64(kwargs)
         if self.process_slug_kwargs():
             # noinspection PyUnresolvedReferences
             return super().dispatch(request, *args, **self.kwargs)
@@ -85,7 +87,8 @@ class BaseModalMixin(AjaxHelpers):
             raise ModalException('User does not have permission')
 
     def button_refresh_modal(self, **_kwargs):
-        return self.command_response(ajax_modal_replace(self.request, modal_class=self.__class__))
+        return self.command_response(ajax_modal_replace(self.request, modal_class=self.__class__,
+                                                        slug=self.kwargs.get('slug', '-')))
 
     def button_group(self):
         button_kwargs = {
@@ -135,6 +138,8 @@ class Modal(BaseModal):
         return self.kwargs.get('message', '')
 
     def get_modal_buttons(self):
+        if 'buttons' in self.kwargs:
+            return self.kwargs['buttons']
         button_group_type = self.kwargs.get('button_group_type')
         if button_group_type == 'confirm':
             return [
@@ -169,13 +174,31 @@ class TemplateModal(Modal):
 
     modal_template = None
 
-    @staticmethod
-    def modal_context():
-        return {}
+    def modal_context(self):
+        context = self.kwargs.get('context', {})
+        return context
 
     def modal_content(self):
         return render_to_string(self.modal_template, self.modal_context())
 
+    def __init__(self, modal_template=None, modal_title=None, size=None, **kwargs):
+        # These kwargs will be overwritten if called as_view()
+        self.kwargs = kwargs
+        if size:
+            self.size = size
+        if modal_title:
+            self.modal_title = modal_title
+        if modal_template:
+            self.modal_template = modal_template
+        super().__init__()
+
+    def modal_html(self, request):
+        self.request = request
+        context = self.get_context_data()
+        if 'message' in self.kwargs:
+            context['message'] = self.kwargs['message']
+        return render_to_string(self.template_name, context)
+
 
 class FormModalMixin(BaseModalMixin):
     template_name = 'django_modals/modal_base.html'
@@ -186,10 +209,7 @@ class FormModalMixin(BaseModalMixin):
             return HttpResponse(str(form))
         return self.refresh_form(form)
 
-    def create_object(self):
-        pass
-
-    def update_object(self):
+    def post_save(self, created):
         pass
 
     def form_valid(self, form):
@@ -197,10 +217,7 @@ class FormModalMixin(BaseModalMixin):
         save_function = getattr(form, 'save', None)
         if save_function:
             save_function()
-        if org_id:
-            self.update_object()
-        else:
-            self.create_object()
+        self.post_save(created=org_id is None)
         if not self.response_commands:
             self.add_command('reload')
         return self.command_response()
@@ -229,7 +246,7 @@ class FormModalMixin(BaseModalMixin):
         self.slug['modal'] = 'editdelete'
         new_slug = '-'.join([f'{k}-{v}' for k, v in self.slug.items()])
         self.request.method = 'GET'
-        self.process = PROCESS_EDIT_DELETE
+        self.process = processes.PROCESS_EDIT_DELETE
         self.request.path = reverse(self.request.resolver_match.url_name, kwargs={'slug': new_slug})
         return self.command_response('overwrite_modal',
                                      html=render_to_string(self.template_name, self.get_context_data()))
@@ -248,6 +265,8 @@ class FormModalMixin(BaseModalMixin):
         kwargs['no_buttons'] = self.request.GET.get('no_buttons')
         if hasattr(self, 'form_setup') and callable(self.form_setup):
             kwargs['form_setup'] = self.form_setup
+        if hasattr(self, 'clean') and callable(self.clean):
+            kwargs['clean'] = self.clean
         kwargs.update({k: getattr(self, k, None) for k in ['modal_title', 'slug']})
         if hasattr(self, 'helper_class'):
             kwargs['helper_class'] = self.helper_class
@@ -306,10 +325,10 @@ class ModelFormModal(SingleObjectMixin, FormModal):
     delete_message = 'Are you sure you want to delete?'
     delete_title = 'Warning'
     field_classes = None
-    permission_delete = PERMISSION_DISABLE
-    permission_edit = PERMISSION_OFF
-    permission_view = PERMISSION_OFF
-    permission_create = PERMISSION_OFF
+    permission_delete = processes.PERMISSION_DISABLE
+    permission_edit = processes.PERMISSION_OFF
+    permission_view = processes.PERMISSION_OFF
+    permission_create = processes.PERMISSION_OFF
 
     @staticmethod
     def formfield_callback(f, **kwargs):
@@ -323,10 +342,8 @@ class ModelFormModal(SingleObjectMixin, FormModal):
             return form_class(**kwargs)
         return f.formfield(**kwargs)
 
-    def __init__(self, *args, **kwargs):
-        self.form_init_args = {}
+    def get_form_class(self):
         if not self.form_class:
-
             processed_form_fields = ProcessFormFields(self.form_fields, widgets=getattr(self, 'widgets', None),
                                                       field_classes=getattr(self, 'field_classes', None),
                                                       labels=getattr(self, 'labels', None),
@@ -337,6 +354,10 @@ class ModelFormModal(SingleObjectMixin, FormModal):
             self.form_class = modelform_factory(self.model, form=self.base_form, fields=processed_form_fields.fields,
                                                 formfield_callback=self.formfield_callback,
                                                 **processed_form_fields.extra_kwargs())
+        return self.form_class
+
+    def __init__(self, *args, **kwargs):
+        self.form_init_args = {}
         super().__init__(*args, **kwargs)
         self.object = None
 
@@ -351,7 +372,7 @@ class ModelFormModal(SingleObjectMixin, FormModal):
         pass
 
     def button_confirm_delete(self, **_kwargs):
-        if self.process in [PROCESS_DELETE, PROCESS_EDIT_DELETE]:
+        if self.process in [processes.PROCESS_DELETE, processes.PROCESS_EDIT_DELETE]:
             self.object.delete()
         self.object_delete()
         if not self.response_commands:
@@ -363,30 +384,68 @@ class ModelFormModal(SingleObjectMixin, FormModal):
         return self.confirm(self.delete_message, self.delete_title, button_function='confirm_delete',
                             button_group_type='yes_cancel', size='md')
 
-    def permission(self, process=None):
-        if process is None:
-            process = self.process
-        perm_value = user_has_perm(self.__class__, self.request.user, process)
-        if not perm_value and process == PROCESS_EDIT:
-            self.process = PROCESS_VIEW
-            perm_value = self.permission(PROCESS_VIEW)
-        return perm_value
+    @staticmethod
+    def user_has_perm(cls_or_instance, user, process):
+        permission_type = getattr(cls_or_instance, processes.process_data[process].class_attribute)
+        if permission_type == processes.PERMISSION_METHOD:
+            # If permission method is not a staticmethod and function is called by class rather than instance
+            # send None instead of self
+            if inspect.isclass(cls_or_instance) and len(inspect.signature(cls_or_instance.permission).parameters) == 3:
+                permission = cls_or_instance.permission(None, user, process)
+            else:
+                permission = cls_or_instance.permission(user, process)
+        elif permission_type == processes.PERMISSION_OFF:
+            permission = True
+        elif permission_type == processes.PERMISSION_DISABLE:
+            permission = False
+        elif permission_type == processes.PERMISSION_AUTHENTICATED:
+            permission = user.is_authenticated
+        elif permission_type == processes.PERMISSION_STAFF:
+            permission = user.is_staff or user.is_superuser
+        else:
+            # noinspection PyProtectedMember
+            perms = [f'{cls_or_instance.model._meta.app_label}.{p}_{cls_or_instance.model._meta.model_name}'
+                     for p in processes.process_data[process].django_permission]
+            permission = user.has_perms(perms)
+        return permission
+
+    def get_process(self, user, process):
+        while True:
+            permission = self.user_has_perm(self, user, process)
+            if permission:
+                break
+            process = processes.process_data[process].fallback
+            if not process:
+                break
+        return permission, process
+
+    def get_model(self):
+        pass
+
+    def get_queryset(self):
+        query = super().get_queryset()
+        if hasattr(self.model, 'query_filter'):
+            return self.model.query_filter(query, self.request, modal=self)
+        return query
 
     def process_slug_kwargs(self):
         if 'pk' not in self.slug:
-            self.process = PROCESS_CREATE
+            self.process = processes.PROCESS_CREATE
         elif 'modal' in self.slug:
-            self.process = modal_url_type[self.slug['modal']]
+            self.process = processes.modal_url_type[self.slug['modal']]
         else:
             if self.process is None:
-                self.process = PROCESS_EDIT_DELETE
-        has_perm, self.process = get_process(self, self.request.user, self.process)
+                self.process = processes.PROCESS_EDIT_DELETE
 
         if self.model is None:
-            self.model = self.form_class.get_model(self.slug)
+            if self.form_class:
+                self.model = self.form_class.get_model(self.slug)
+            else:
+                self.model = self.get_model()
         if 'pk' in self.kwargs:
             self.object = self.get_object()
         else:
+            self.initial = self.initial.copy()
             self.object = self.model()
             # noinspection PyProtectedMember
             fields = self.model._meta.get_fields()
@@ -398,6 +457,7 @@ class ModelFormModal(SingleObjectMixin, FormModal):
                     self.initial[i] = [self.slug[i]]
                 else:
                     setattr(self.object, i, self.slug[i])
+        has_perm, self.process = self.get_process(self.request.user, self.process)
         return has_perm
 
     def select2_ajax_search(self, page_len=10, filter_field=None, filter_search='istartswith', search=None, page=None,
@@ -426,8 +486,49 @@ class ModelFormModal(SingleObjectMixin, FormModal):
         return JsonResponse({'results': results, 'pagination': {'more': len(choices) > len(results)}})
 
 
-MultiForm = collections.namedtuple('MultiForm', ['model', 'fields', 'id', 'initial', 'widgets'],
-                                   defaults=[None, None, None, {}, []])
+class MultiForm:
+
+    def __init__(self, model, fields, form_id=None, initial=None, widgets=None, pk=None, **kwargs):
+        self.model = model
+        self.fields = fields
+        self.kwargs = kwargs
+        self.form_id = form_id
+        self.initial = initial if initial else {}
+        self.widgets = widgets if widgets else {}
+        self.pk = pk
+
+    def make_form_id(self, used_ids):
+        if not self.form_id:
+            self.form_id = self.model.__name__ + 'Form'
+            if self.form_id in used_ids:
+                self.form_id += '_{}'
+                count = 1
+                while self.form_id.format(count) in used_ids:
+                    count += 1
+                self.form_id = self.form_id.format(count)
+        used_ids.append(self.form_id)
+
+    def get_kwargs(self):
+        kwargs = {'form_id': self.form_id, 'initial': self.initial, 'no_buttons': True}
+        if self.pk:
+            kwargs.update({'instance': self.model.objects.get(pk=self.pk)})
+        kwargs.update(self.kwargs)
+        return kwargs
+
+
+class DictGetList(dict):
+    """
+    Adds getlist to dict to make dict work more like Django's MultiValueDict
+    """
+    def getlist(self, key, default=None):
+        try:
+            values = super().__getitem__(key)
+        except KeyError:
+            if default is None:
+                return []
+            return default
+        else:
+            return values if isinstance(values, list) else [values]
 
 
 class MultiFormModal(BaseModal):
@@ -451,38 +552,31 @@ class MultiFormModal(BaseModal):
         super().__init__(*args, **kwargs)
         self.form_setup_args = []
 
-    @staticmethod
-    def make_form_id(form, used_ids):
-        form_id = form.model.__name__ + 'Form'
-        if form_id not in used_ids:
-            return form_id
-        form_id += '_{}'
-        count = 1
-        while form_id.format(count) in used_ids:
-            count += 1
-        return form_id.format(count)
-
     def get_form_kwargs(self):
         all_kwargs = []
         used_ids = []
         if self.request.method in ('POST', 'PUT'):
             form_data = json.loads(self.request.body)
+            for f in form_data:
+                if isinstance(form_data[f], dict):
+                    form_data[f] = DictGetList(**form_data[f])
         else:
             form_data = {}
         for f in self.forms:
-            form_id = f.id
-            if not form_id:
-                form_id = self.make_form_id(f, used_ids)
-            used_ids.append(form_id)
-            kwargs = f.initial.copy()
-            kwargs['form_id'] = form_id
-            kwargs['no_buttons'] = True
+            f.make_form_id(used_ids)
+            kwargs = f.get_kwargs()
             if self.request.method in ('POST', 'PUT'):
                 kwargs.update({
-                    'data': form_data[form_id],
+                    'data': form_data[f.form_id],
                     # 'files': self.request.FILES,
                 })
-            if hasattr(self, 'form_setup') and callable(self.form_setup):
+            if hasattr(f, 'clean') and callable(f.clean):
+                kwargs['clean'] = f.clean
+            elif hasattr(self, 'clean') and callable(self.clean):
+                kwargs['clean'] = self.clean
+            if hasattr(f, 'form_setup') and callable(f.form_setup):
+                kwargs['form_setup'] = f.form_setup
+            elif hasattr(self, 'form_setup') and callable(self.form_setup):
                 kwargs['form_setup'] = self.form_setup
             all_kwargs.append(kwargs)
         all_kwargs[-1]['no_buttons'] = False
@@ -495,7 +589,10 @@ class MultiFormModal(BaseModal):
         for c, s in enumerate(self.form_setup_args):
             kwargs = form_kwargs[c]
             kwargs.update(s['processed_form_fields'].form_init_kwargs())
-            forms.append(s['form_class'](**kwargs))
+            form = s['form_class'](**kwargs)
+            for field_name, field in form.fields.items():
+                field.widget.attrs.update({'id': f'id_{c}_{field_name}'})
+            forms.append(form)
         return forms
 
     def get_context_data(self, **kwargs):
