@@ -54,10 +54,11 @@ if (typeof django_modal == 'undefined') {
             show_modal(command.modal);
         }
         ajax_helpers.command_functions.modal_html = function (command) {
+            ajax_helpers.ajax_busy = true
             create_modal(command.html);
         }
         ajax_helpers.command_functions.post_modal = function (command) {
-            send_inputs(command.button);
+            send_inputs(command.button,command.options);
         }
 
         ajax_helpers.command_functions.modal_refresh_trigger = function (command) {
@@ -65,8 +66,6 @@ if (typeof django_modal == 'undefined') {
             $(command.selector).trigger('modalPostLoad', [django_modal.active_modal_container_id()])
         }
 
-
-
         if (window.opener != null) {
             target = window.opener.location.href
         }
@@ -112,20 +111,28 @@ if (typeof django_modal == 'undefined') {
         }
 
         function disable_enter_key(){
-            $('form input').keydown(function (e) {
+            $('input', modal_div()).keydown(function (e) {
                 if (e.keyCode === 13) {
                     e.preventDefault();
-                    $('.modal-submit')[0].click()
+                    var form_no_enter = $('form', modal_div()).attr('no-enter')
+                    var flag_no_enter = (form_no_enter !== undefined) && (form_no_enter !== false)
+                    if ($(this).hasClass('no-enter') || flag_no_enter){
+                        var el = $('input, button, select', modal_div())
+                        el[el.index(this) + 1].focus()
+                    } else {
+                        $('.modal-submit')[0].click()
+                    }
                     return false;
                 }
             });
         }
 
         function init_modal_container(modal_container){
+            var left_pos;
             var modal_element = modal_container.children();
             modal_element.css('z-index', 1040 + (10 * open_modals));
             modal_element.modal({'backdrop': false})
-            modal_dialog = $('.modal-dialog', modal_element).first()
+            var modal_dialog = $('.modal-dialog', modal_element).first()
             if ((document.documentElement.clientWidth - modal_dialog.width()) > 20){
                 left_pos = open_modals*10 - 20
             } else {
@@ -165,8 +172,8 @@ if (typeof django_modal == 'undefined') {
 
         function additional_parameters(params) {
             if (typeof (params) == 'undefined') params = {};
-            params.modal_id = active_modal_container_id()
-            //params.modal_page_url = window.location.pathname;
+            params.modal_id = active_modal_container_id();
+            params.modal_page_url = window.location.pathname;
             //params.modal_querystring = window.location.search;
             return params;
         }
@@ -216,7 +223,7 @@ if (typeof django_modal == 'undefined') {
             })
         }
 
-        function send_inputs(button_name, callback, index) {
+        function send_inputs(button_name, options) {
             var params
             var property
             if (typeof (button_name) == 'object') {
@@ -233,9 +240,6 @@ if (typeof django_modal == 'undefined') {
             if (modal_type !== undefined){
                 params['modal_type'] = modal_type
             }
-            if (typeof callback != 'undefined') {
-                modal_url = url_change(modal_url, 'modalstyle', 'windowform')
-            }
             if (typeof (tinymce) != 'undefined') tinymce.triggerSave();
 
             var forms = modalDiv.find('form')
@@ -245,7 +249,14 @@ if (typeof django_modal == 'undefined') {
                     var form_data = new FormData(f)
                     data[f.id] = {}
                     for (var d of form_data.entries()) {
-                        data[f.id][d[0]] = d[1]
+                        if (Array.isArray(data[f.id][d[0]])) {
+                            data[f.id][d[0]].push(d[1])
+                        } else if (data[f.id][d[0]] != undefined)
+                        {
+                            data[f.id][d[0]] = [data[f.id][d[0]], d[1]]
+                        } else{
+                            data[f.id][d[0]] = d[1]
+                        }
                     }
                 }
                 for (property in params) {
@@ -258,7 +269,7 @@ if (typeof django_modal == 'undefined') {
                 for (property in params) {
                     data.append(property, params[property])
                 }
-                ajax_helpers.post_data(modal_url, data, django_modal.timeout)
+                ajax_helpers.post_data(modal_url, data, django_modal.timeout, options)
             }
         }
 
@@ -330,7 +341,11 @@ if (typeof django_modal == 'undefined') {
                 if (e !== undefined) {
                     django_modal.send_inputs(config.button)
                 }
+            },
+            html: function (config, e, value) {
+                $(config.selector).html(value);
             }
+
         }
 
         var modal_triggers = {}
@@ -384,13 +399,25 @@ if (typeof django_modal == 'undefined') {
         }
 
         function perform_function(config, value, e) {
+            var parameters;
             var html_function = config.values[value]
+
             if (html_function === undefined) {
-                if (config.default !== undefined) {
-                    form_change_functions[config.default](config, e)
+                var default_config = config.default;
+
+                if (default_config !== undefined) {
+                    if (default_config instanceof Array) {
+                        parameters = default_config[1]
+                        default_config = default_config[0]
+                    }
+                    form_change_functions[default_config](config, e, parameters)
                 }
             } else {
-                form_change_functions[html_function](config, e)
+                if (html_function instanceof Array) {
+                    parameters = html_function[1]
+                    html_function = html_function[0]
+                }
+                form_change_functions[html_function](config, e, parameters)
             }
         }
 
