@@ -24,17 +24,22 @@ class SelectGroupMixin:
         return context
 
 
+def add_width(style):
+    no_space_style = style.replace(' ', '')
+    if not style:
+        return 'width:100%'
+    elif not no_space_style.startswith('width:') and ';width:' not in no_space_style:
+        return style + ';width:100%'
+    return style
+
+
 class Select2(SelectGroupMixin, Select):
     template_name = 'django_modals/widgets/select2.html'
 
     def get_context(self, name, value, attrs):
         context = super().get_context(name, value, attrs)
-        existing_style = context['widget']['attrs'].get('style', '')
-        if not existing_style:
-            context['widget']['attrs']['style'] = 'width:100%'
-        elif 'width' not in existing_style:
-            context['widget']['attrs']['style'] = existing_style + ';width:100%'
-        if not self.attrs.get('ajax') and hasattr(self, 'select_data'):
+        context['widget']['attrs']['style'] = add_width(context['widget']['attrs'].get('style', ''))
+        if hasattr(self, 'select_data'):
             context['widget']['select_data'] = mark_safe(json.dumps(self.select_data))
         return context
 
@@ -53,6 +58,11 @@ class Select2Multiple(SelectGroupMixin, SelectMultiple):
             kwargs.setdefault('attrs', {}).update({'new_marker': self.new_marker, 'tags': True})
         super().__init__(*args, **kwargs)
 
+    def get_context(self, name, value, attrs):
+        context = super().get_context(name, value, attrs)
+        context['widget']['attrs']['style'] = add_width(context['widget']['attrs'].get('style', ''))
+        return context
+
 
 class ModelSelect2ChoiceField(ChoiceField):
 
@@ -96,3 +106,30 @@ class MultipleChoiceFieldAddValues(MultipleChoiceField):
 
     def widget_attrs(self, widget):
         return {'new_marker': Select2Multiple.new_marker, 'tags': True}
+
+
+class ChoiceFieldAddValues(ChoiceField):
+
+    new_marker = 'new:'
+
+    def __init__(self, choices=None, model=None, display_field=None, widget=None, **kwargs):
+        if model and display_field:
+            choices = model.objects.values_list('id', display_field)
+        if not widget:
+            self.widget = Select2(attrs={'new_marker': 'new:', 'tags': True})
+        super().__init__(choices=choices, **kwargs)
+
+    def clean(self, value):
+        return value
+
+    def bound_data(self, data, initial):
+        if self.new_data(data):
+            if self.choices[-1][1] != self.new_data(data):
+                self.choices.append((data, self.new_data(data)))
+        return super().bound_data(data,initial)
+
+    @classmethod
+    def new_data(cls, value):
+        if value.startswith(cls.new_marker):
+            return value[len(cls.new_marker):]
+
